--with Ada.Text_Io;

--use Ada.Text_Io;

package body Paskma.Hash_Tables is

   ----------------
   -- Get_Values --
   ----------------

   procedure Get_Values_Access (
         Self   : in     Hash_Table;         
         Key    : in     Key_Type;           
         Values :    out Value_List_Accesses ) is 

      Index : Natural;  
      I,  
      I_End : Cell_Lists.Iterator;  
      Found : Boolean;  
   begin
      Values := null;
      Index := Hash_Function(Key) mod Self.Modulo;

      if Self.Table(Index) = null then
         raise Key_Not_Found;
      end if;

      --
      -- Walk through the slot and ty the found the appropriate key
      -- if it is found then insert the value
      I := Get_Begin(Self.Table(Index).all);
      I_End := Get_End(Self.Table(Index).all);

      Found := False;
      while I /= I_End loop
         if Access_Dereference(I).Key = Key then
            Values := Access_Dereference(I).Values'access;
            Found := True;
            exit;
         end if;
         Increment(I);
      end loop;

      if not Found then
         raise Key_Not_Found;
      end if;

   end Get_Values_Access;

   ------------
   -- Insert --
   ------------

   procedure Insert (
         Self  : in out Hash_Table; 
         Key   :        Key_Type;   
         Value :        Value_Type  ) is 

      Index : Natural;  
      I,  
      I_End : Cell_Lists.Iterator;  
      Added : Boolean;  
   begin
      Index := Hash_Function(Key) mod Self.Modulo;

      --
      -- If the appropriate slot is not created then create it
      if Self.Table(Index) = null then
         Self.Table(Index) := new Cell_Lists.List;
      end if;

      --
      -- Walk through the slot and ty the found the appropriate key
      -- if it is found then insert the value
      I := Get_Begin(Self.Table(Index).all);
      I_End := Get_End(Self.Table(Index).all);

      Added := False;
      while I /= I_End loop
         if Access_Dereference(I).Key = Key then
            Push_Back(Value, Access_Dereference(I).Values);
            Added := True;
            exit;
         end if;
         Increment(I);
      end loop;

      --
      -- There is not valid key yet. So create a new cell and insert the value
      if not Added then
         declare
            New_Cell : Cell_Type;  
         begin
            New_Cell.Key := Key;
            Push_Back(Value, New_Cell.Values);
            Push_Back(New_Cell, Self.Table(Index).all);
         end;
      end if;

   end Insert;

   --
   -- Get_Keys
   --

   procedure Get_Keys (
         Self   : in     Hash_Table;    
         Result : in out Key_Lists.List ) is 

      Iter,  
      Iter_End : Cell_Lists.Iterator;  
   begin
      Empty (Result );

      for I in Self.Table.All'range loop

         if Self.Table.All(I) /= null then

            --Put_Line(Natural'Image(Length(Self.Table(I).all)));

            Iter := Get_Begin(Self.Table(I).all);
            Iter_End := Get_End(Self.Table(I).all);
            --
            -- Add the keys in the actual nonempty slot
            while Iter /= Iter_End loop
               Push_Back(Access_Dereference(Iter).Key, Result);
               Increment(Iter);
            end loop;

            --else
            --   Put_Line("0");
         end if;
      end loop;
   end;

   ----------------
   -- Set_Modulo --
   ----------------

   procedure Set_Modulo (
         Self   : in out Hash_Table; 
         Modulo :        Positive    ) is 
   begin
      --
      -- Really unsafe pre-implementation
      Self.Table := new Cell_Table_Type(0..Modulo-1);

      for I in Self.Table'range loop
         Self.Table(I) := null;
      end loop;

      Self.Modulo := Modulo;
   end Set_Modulo;

   --
   --  Controlled stuff
   --

   procedure Initialize (
         Self : in out Hash_Table ) is 
   begin
      Self.Modulo := 1;
      Self.Table := null;
   end;

   Not_Implemented : exception;  
   procedure Adjust (
         Self : in out Hash_Table ) is 
   begin
      raise Not_Implemented;
   end;


   procedure Finalize (
         Self : in out Hash_Table ) is 
   begin
      for I in Self.Table'range loop
         Free(Self.Table(I));
      end loop;

      Free(Self.Table);
   end;

end Paskma.Hash_Tables;

