with Ada.Characters.Handling;

use Ada.Characters.Handling;

package body Paskma.String_Utilities is

   -------------------------
   -- Insensitive_Compare --
   -------------------------

   function Insensitive_Compare (
         Left,                    
         Right : Unbounded_String ) 
     return Boolean is 

      Internal_Left  : constant String := To_Upper (To_String (Left));  
      Internal_Right : constant String := To_Upper (To_String (Right));  
   begin
      return Internal_Left = Internal_Right;
   end Insensitive_Compare;

   ----------------------------------------------------------------------------------

   function Insensitive_Compare (
         Left,          
         Right : String ) 
     return Boolean is 

      Internal_Left  : constant String := To_Upper (Left);  
      Internal_Right : constant String := To_Upper (Right);  
   begin
      return Internal_Left = Internal_Right;
   end Insensitive_Compare;

   ----------------------------------------------------------------------------------

   function Insensitive_Compare (
         Left  : Unbounded_String; 
         Right : String            ) 
     return Boolean is 

      Internal_Left  : constant String := To_Upper (To_String (Left));  
      Internal_Right : constant String := To_Upper (Right);  
   begin
      return Internal_Left = Internal_Right;
   end Insensitive_Compare;

   ----------------------------------------------------------------------------------

   function Insensitive_Compare (
         Left  : String;          
         Right : Unbounded_String ) 
     return Boolean is 

      Internal_Left  : constant String := To_Upper (Left);  
      Internal_Right : constant String := To_Upper (To_String (Right));  
   begin
      return Internal_Left = Internal_Right;
   end Insensitive_Compare;

   ----------------------------------------------------------------------------------

   --
   -- Hash
   --

   function Hash (
         S : Unbounded_String ) 
     return Integer is 
      --    public int hashCode() {
      --        int h = hash;
      --        if (h == 0) {
      --            int off = offset;
      --            char val[] = value;
      --            int len = count;

      --            for (int i = 0; i < len; i++) {
      --                h = 31*h + val[off++];
      --            }
      --            hash = h;
      --        }
      --        return h;
      --    }
      type Round_Type is mod Integer'Last; 
      H : Round_Type := 0;  
   begin

      for I in 1..Length(S) loop
         H := 31*H + Character'Pos(Element(S, I));
      end loop;

      return Integer(H);
   end Hash;

   --
   -- Get_Elements
   --
   procedure Get_Elements (
         S         :        Unbounded_String; 
         Delimiter :        Character;        
         Result    : in out List              ) is 

      Source : String := To_String (S);  

      Start,  
      Stop  : Natural;  

      procedure Insert (
            A : String ) is 
      begin
         if A'Length = 0 then
            return;
         end if;

         Push_Back(To_Unbounded_String(A), Result);
      end Insert;

   begin
      Empty(Result);
      Start := Source'First-1;
      Stop := Source'First-1;

      while Stop < Source'Last loop

         loop
            Stop := Stop+1;
            exit when Stop = Source'Last or Source(Stop) = Delimiter;
         end loop;

         if Stop = Source'Last and Source(Stop) /= Delimiter  then
            Stop := Stop + 1;
         end if;

         Insert(Source(Start+1..Stop-1));
         Start := Stop;

      end loop;

   end Get_Elements;

   --
   -- +
   --

   function "+" (
         Source : String ) 
     return Unbounded_String is 
   begin
      return To_Unbounded_String(Source);
   end;
   pragma Inline("+");

   --
   -- +
   --

   function "+" (
         Source : Unbounded_String ) 
     return String is 
   begin
      return To_String(Source);
   end;
   Pragma Inline("+");


end Paskma.String_Utilities;

