with Ada.Strings.Unbounded,
   Paskma.Logger;

use Ada.Strings.Unbounded,
   Paskma.Logger;

package body Configuration_Mycelium is

   -------------
   -- Get_Key --
   -------------

   function Get_Key (
         Line : Unbounded_String ) 
     return Unbounded_String is 

      Equal_Index : Natural;  
   begin
      Equal_Index := Index(Line, "=");

      if Equal_Index = 0 then
         raise Invalid_Line;
      end if;

      return To_Unbounded_String(Slice(Line, 1, Equal_Index-1));
   end Get_Key;

   ---------------
   -- Get_Value --
   ---------------

   function Get_Value (
         Line : Unbounded_String ) 
     return Unbounded_String is 

      Equal_Index : Natural;  
   begin
      Equal_Index := Index(Line, "=");

      if Equal_Index = 0 then
         raise Invalid_Line;
      end if;

      if Equal_Index = Length(Line) then
         -- there is no char behind equal sign
         return To_Unbounded_String("");
      else
         return To_Unbounded_String(Slice(Line, Equal_Index+1, Length(Line)));
      end if;

   end Get_Value;

   --
   -- Get_Line
   --

   function Get_Line (
         File : File_Type ) 
     return Unbounded_String is 

      Buffer : String (1 .. 1024);  
      Last   : Natural;  
   begin
      loop
         Get_Line(File, Buffer, Last);

         if Last = 0 or else Buffer(1) = '#' then
            null;
            -- skip empty lines and commentaries
         else
            return To_Unbounded_String(Buffer(1..Last));
         end if;

      end loop;

   exception
      when End_Error =>
         return To_Unbounded_String("");
   end Get_Line;

   --
   -- Log
   --

   procedure Log (
         User_Configuration : User_Configuration_Type ) is 

      use Shared_Dir_Lists;
      I     : Iterator;  
      I_End : Iterator;  
   begin
      Log("Username = " & User_Configuration.Username);
      Log("Password = " & User_Configuration.Password);
      Log("Root directory = " & User_Configuration.Root_Directory);
      Log("List of accessible directories = ");

      I := Get_Begin(User_Configuration.Directories);
      I_End := Get_End(User_Configuration.Directories);
      while I /= I_End loop

         Log("Name =  " & Dereference(I).Name);
         Log("Rights = " & To_String(Dereference(I).Rights));

         Increment(I);
      end loop;
   end;

   --
   -- Log
   --

   procedure Log (
         Mapping : Mapping_Type ) is 
   begin
      Log("Directory = " & Mapping.Directory);
      Log("Alias = " & Mapping.Alias);
   end;


end Configuration_Mycelium;

