with Gnat.Calendar.Time_Io,
   Ada.Strings.Fixed,
   Ada.Calendar;

use Gnat.Calendar.Time_Io,
   Ada.Strings.Fixed,
   Ada.Calendar;

package body Directory_Listing_Utilities is


   --
   -- To_String
   --

   function To_String (
         Attributes : File_Attribute_Set ) 
     return Unbounded_String is 

      Normal_Template    : constant String           := "?rw-rw-rw-";  
      Read_Only_Template : constant String           := "?r--r--r--";  
      Result             :          Unbounded_String;  
   begin
      if Attributes(Read_Only) then
         --the file is read only
         Result := To_Unbounded_String(Read_Only_Template);
      else
         -- file is NOT read only
         Result := To_Unbounded_String(Normal_Template);
      end if;

      if Attributes(Directory) then
         -- the file is a directory
         Replace_Element(Result, 1, 'd');
      else
         -- file is a file (really!)
         Replace_Element(Result, 1, '-');
      end if;

      return Result;
   end To_String;

   --
   -- To_Vdir_String
   --

   function To_Vdir_String (
         Point : Time ) 
     return Unbounded_String is 

      Now : constant Time := Clock;  
   begin
      if Year(Now) /= Year(Point) then
         --this is not very actual point => display year instad of hour and minute
         return To_Unbounded_String(Image(Point, "%b %d  %Y"));
      else
         -- this is actual date => display exact time
         return To_Unbounded_String(Image(Point, "%b %d %H:%M"));
      end if;
   end To_Vdir_String;


   ---------------
   -- To_String --
   ---------------

   function To_String (
         Item : Directory_Item_Type ) 
     return Unbounded_String is 
   begin

      return To_String(Item.Attributes) & "  1"
         & " no_user"
         & " no_group"
         & Tail(Long_Long_Integer'Image(Item.Size), 12)
         &  "   "
         & To_Vdir_String(Item.Creation_Time)
         & " "
         -- here must be only one space because of some silly FTP clients
         & Item.Name;

   end To_String;
   
end Directory_Listing_Utilities;

