with
   System,
   Win32.Winnt,
   Win32.Winbase,
   Ada.Exceptions,
   Win32,
   --Ada.Characters.Latin_1,
   --Ada.Strings.Fixed,
   System.Bit_Ops,
   File_Utilities,
   Paskma.Logger,
   C_Conversions,
   Ada.Unchecked_Conversion,
   Gnat.Directory_Operations,
   Gnat.Os_Lib;

use System,
   Win32,
   Win32.Winbase,
   Ada.Exceptions,
   --Ada.Strings.Fixed,
   System.Bit_Ops,
   File_Utilities,
   Paskma.Logger,
   C_Conversions,
   Win32.Winnt,
   Gnat.Directory_Operations,
   Gnat.Os_Lib;

package body Directory_Listings is

   use Directory_Item_Lists;

   --------------------------------------------------------------------------------

   function To_Directory_Item (
         Found : Win32_Find_Data ) 
     return Directory_Item_Type is 

      Result          : Directory_Item_Type;  
      Tmp_Size_Low,  
      Tmp_Size_High   : Long_Long_Integer;  
      Bitresult,  
      Tmp_Attribute1,  
      Tmp_Attribute2  : aliased Win32.Dword;  
      Tmp_Source_Time : aliased Filetime;  
      Tmp_Result_Time : aliased Systemtime;  
      Suc             : Win32.Bool;  
   begin
      --
      -- File Name
      Result.Name := To_Unbounded_String(To_String(Found.Cfilename));
      --
      -- File Size
      Tmp_Size_Low := Long_Long_Integer(Found.Nfilesizelow);
      Tmp_Size_High := Long_Long_Integer(Found.Nfilesizehigh);
      Result.Size := Tmp_Size_High * 2**32 + Tmp_Size_Low;
      --
      -- Attributes
      for I in Result.Attributes'range loop
         Result.Attributes(I) := False;
      end loop;
      --
      -- directory
      Tmp_Attribute1 := File_Attribute_Directory;
      Tmp_Attribute2 :=  Found.Dwfileattributes;

      Bit_And(Tmp_Attribute1'Address, Tmp_Attribute1'Size,
         Tmp_Attribute2'Address, Tmp_Attribute2'Size,
         Bitresult'Address);

      if Integer(Bitresult) /= 0 then
         Result.Attributes(Directory) := True;
      end if;
      --
      -- readonly
      Tmp_Attribute1 := File_Attribute_Readonly;
      Tmp_Attribute2 :=  Found.Dwfileattributes;

      Bit_And(Tmp_Attribute1'Address, Tmp_Attribute1'Size,
         Tmp_Attribute2'Address, Tmp_Attribute2'Size,
         Bitresult'Address);

      if Integer(Bitresult) /= 0 then
         Result.Attributes(Read_Only) := True;
      end if;

      --
      -- Creation time
      Tmp_Source_Time := Found.Ftlastwritetime;
      Suc := Filetimetosystemtime(Tmp_Source_Time'Unchecked_Access,
         Tmp_Result_Time'Unchecked_Access);

      if Suc = 0 then
         raise Listing_Error;
      end if;

      Result.Creation_Time := Time_Of(
         Year    => Year_Number (Tmp_Result_Time.Wyear),                                                                                               
         Month   => Month_Number (Tmp_Result_Time.Wmonth),                                                                                             
         Day     => Day_Number (Tmp_Result_Time.Wday),                                                                                                 
         Seconds => Day_Duration (Integer (Tmp_Result_Time.Whour) * 3600 + Integer (Tmp_Result_Time.Wminute) * 60 + Integer (Tmp_Result_Time.Wsecond))
         );

      return Result;
   end;

   --------------------------------------------------------------------------------


   -----------------
   -- Get_Listing --
   -----------------

   procedure Get_Listing0 (
         Directory_Name  :        Unbounded_String;                 
         Result          : in out Directory_Item_Lists.List;        
         Attach_Asterisk :        Boolean                   := True ) is 

      Listing_Handle          : Handle;  
      Internal_Directory_Name : Unbounded_String;  
      Found                   : aliased Win32_Find_Data;  
      Usuable_Found           : Directory_Item_Type;  
      Suc                     : Bool;  
   begin
      Empty(Result);

      if Attach_Asterisk then
         Internal_Directory_Name := Compose_Path(Directory_Name,
            To_Unbounded_String("*"&Character'First));
      else
         -- this is usuable when we want determine attributes of given filename only
         Internal_Directory_Name := Directory_Name & Character'First;
         --attach '\0'
      end if;

      Log("Get_Listing for " & Internal_Directory_Name);

      Listing_Handle := Findfirstfile(To_Lpcstr(To_String(
               Internal_Directory_Name)),
         Found'Unchecked_Access);

      if Listing_Handle = Invalid_Handle_Value then
         -- probaly not valid filename was given
         --raise Listing_Error;
         Raise_Exception(Listing_Error'Identity, "Probably invalid filename " & To_String(Directory_Name));
      end if;

      loop
         Usuable_Found := To_Directory_Item(Found);
         if Usuable_Found.Name /= "." and Usuable_Found.Name /= ".." then
            -- add to the list only if it is the interesting filename
            Push_Back(Usuable_Found, Result);
         end if;
         Suc := Findnextfile(Listing_Handle, Found'Unchecked_Access);
         exit when Suc = 0;
      end loop;

      Suc := Findclose(Listing_Handle);
      if Suc = 0 then
         raise Listing_Error;
      end if;

   end;
   -----------------------------------------------------------
   procedure Get_Attributes (
         Name :        Unbounded_String;   
         Item : in out Directory_Item_Type ) is 

      Fd       : File_Descriptor;  
      Creation : Os_Time;  
   begin
      Log("getting attrib for " & Name);
      Item.Attributes(Read_Only) := False;
      Item.Attributes(Directory) := Is_Directory(To_String(Name));

      Fd := Open_Read(To_String(Name), Binary);
      Item.Size := Long_Long_Integer(File_Length(Fd));
      Creation := File_Time_Stamp(Fd);
      Close(Fd);

      if Gm_Year (Creation) <= 1980 then
         Item.Creation_Time := Time_Of(
            Year  => Year_Number (1980), 
            Month => Month_Number (1),   
            Day   => Day_Number (1)
            );
      else
         Item.Creation_Time := Time_Of(
            Year    => Year_Number (Gm_Year (Creation)),                                                                                         
            Month   => Month_Number (Gm_Month (Creation)),                                                                                       
            Day     => Day_Number (Gm_Day (Creation)),                                                                                           
            Seconds => Day_Duration (Integer (Gm_Hour (Creation)) * 3600 + Integer (Gm_Minute (Creation)) * 60 + Integer (Gm_Second (Creation)))
            );
      end if;

   end;


   procedure Get_Listing (
         Directory_Name  :        Unbounded_String;                 
         Result          : in out Directory_Item_Lists.List;        
         Attach_Asterisk :        Boolean                   := True ) is 

      Dir                     : Dir_Type;  
      Buffer                  : String (1 .. 256);  
      Buffer_Last             : Natural;  
      Usuable_Found           : Directory_Item_Type;  
      Internal_Directory_Name : Unbounded_String;  
   begin
      Empty(Result);
      Log("GET_LISTING CALLED WITH PARAMWRWE " & Directory_Name);

      if Attach_Asterisk then
         Internal_Directory_Name := Compose_Path(Directory_Name,
            To_Unbounded_String("*"));
      else
         -- this is usuable when we want determine attributes of given filename only
         Get_Attributes(Directory_Name, Usuable_Found);
         Usuable_Found.Name := Get_Last_Physical_Path_Element(Directory_Name);
         Push_Back(Usuable_Found, Result);
         return;
      end if;

      Internal_Directory_Name := Directory_Name;


      Open(Dir, To_String(Internal_Directory_Name));
      Log("dir opended");
      loop
         Read(Dir, Buffer, Buffer_Last);
         Log("read called");
         exit when Buffer_Last = 0;
         Log("name returned '" & Buffer(1..Buffer_Last) & "'");
         if Buffer(1..Buffer_Last) /= "." and Buffer(1..Buffer_Last) /= ".." then
            Usuable_Found.Name := To_Unbounded_String(Buffer(1..Buffer_Last));
            Get_Attributes(Compose_Path(Directory_Name, To_Unbounded_String(Buffer(1..Buffer_Last))), Usuable_Found);

            Log("item_pushed");
            Push_Back(Usuable_Found, Result);
         end if;

      end loop;
      Close(Dir);
      Log("dir closed");
   exception
      when
            Directory_Error =>
         Raise_Exception(Listing_Error'Identity, "Probably invalid filename " & To_String(Directory_Name) & " & " & To_String(Internal_Directory_Name));
   end;

   --
   -- Is_Directory
   --

   --   function Is_Directory (
   --         Name : String ) 
   --     return Boolean is 

   --      --Invalid_File_Attributes : constant Win32.Dword         := Win32.Dword (16#ffffffff#);  
   --      Result_Attribute,  
   --      Mask_Attribute,  
   --      Bitresult        : aliased Win32.Dword;  
   --      Internal_Name    : String (Name'First .. Name'Last + 1) := Name & Character'First;  

   --      function Convert is 
   --      new Ada.Unchecked_Conversion(Win32.Dword, Integer);
   --   begin

   --      Result_Attribute := Getfileattributes(To_Lpcstr(Internal_Name));
   --      Mask_Attribute := File_Attribute_Directory;

   --      if Convert(Result_Attribute) = -1 then
   --         return False;
   --      end if;

   --      Bit_And(Result_Attribute'Address, Result_Attribute'Size,
   --         Mask_Attribute'Address, Mask_Attribute'Size,
   --         Bitresult'Address);

   --      if Integer(Bitresult) = 0 then
   --         return False;
   --      else
   --         return True;
   --      end if;

   --   end;

   function Is_Directory (
         Name : String ) 
     return Boolean is 
   begin
      return Gnat.Os_Lib.Is_Directory(Name);
   end;


end Directory_Listings;

