--
-- This is cross-platform inefective version of directory listing
--

with
   Ada.Exceptions,
   File_Utilities,
   Paskma.Logger,
   Gnat.Directory_Operations,
   Gnat.Os_Lib;

use
   Ada.Exceptions,
   File_Utilities,
   Paskma.Logger,
   Gnat.Directory_Operations,
   Gnat.Os_Lib;

package body Directory_Listings is

   use Directory_Item_Lists;

   --
   -- Get_Attributes
   --

   procedure Get_Attributes (
         Name :        Unbounded_String;   
         Item : in out Directory_Item_Type ) is 

      Fd       : File_Descriptor;  
      Creation : Os_Time;  
   begin
      --Log("getting attrib for " & Name);
      Item.Attributes(Read_Only) := False;
      Item.Attributes(Directory) := Is_Directory(To_String(Name));

      Fd := Open_Read(To_String(Name), Binary);
      Item.Size := Long_Long_Integer(File_Length(Fd));
      Creation := File_Time_Stamp(Fd);
      Close(Fd);

      if Gm_Year (Creation) <= 1980 then
         Item.Creation_Time := Time_Of(
            Year  => Year_Number (1980), 
            Month => Month_Number (1),   
            Day   => Day_Number (1)
            );
      else
         Item.Creation_Time := Time_Of(
            Year    => Year_Number (Gm_Year (Creation)),                                                                                         
            Month   => Month_Number (Gm_Month (Creation)),                                                                                       
            Day     => Day_Number (Gm_Day (Creation)),                                                                                           
            Seconds => Day_Duration (Integer (Gm_Hour (Creation)) * 3600 + Integer (Gm_Minute (Creation)) * 60 + Integer (Gm_Second (Creation)))
            );
      end if;

   end;

   -----------------
   -- Get_Listing --
   -----------------

   procedure Get_Listing (
         Directory_Name  :        Unbounded_String;                 
         Result          : in out Directory_Item_Lists.List;        
         Attach_Asterisk :        Boolean                   := True ) is 

      Dir                     : Dir_Type;  
      Buffer                  : String (1 .. 256);  
      Buffer_Last             : Natural;  
      Usuable_Found           : Directory_Item_Type;  
      Internal_Directory_Name : Unbounded_String;  
   begin
      Empty(Result);
      --Log("GET_LISTING CALLED WITH PARAMETER " & Directory_Name);

      if Attach_Asterisk then
         Internal_Directory_Name := Compose_Path(Directory_Name,
            To_Unbounded_String("*"));
      else
         -- this is usuable when we want determine attributes of given filename only
         Get_Attributes(Directory_Name, Usuable_Found);
         Usuable_Found.Name := Get_Last_Physical_Path_Element(Directory_Name);
         Push_Back(Usuable_Found, Result);
         return;
      end if;

      Internal_Directory_Name := Directory_Name;


      Open(Dir, To_String(Internal_Directory_Name));
      --Log("dir opended");
      loop
         Read(Dir, Buffer, Buffer_Last);
         --Log("read called");
         exit when Buffer_Last = 0;
         --Log("name returned '" & Buffer(1..Buffer_Last) & "'");
         if Buffer(1..Buffer_Last) /= "." and Buffer(1..Buffer_Last) /= ".." then
            Usuable_Found.Name := To_Unbounded_String(Buffer(1..Buffer_Last));
            Get_Attributes(Compose_Path(Directory_Name, To_Unbounded_String(Buffer(1..Buffer_Last))), Usuable_Found);

            --Log("item_pushed");
            Push_Back(Usuable_Found, Result);
         end if;

      end loop;
      Close(Dir);
      --      Log("dir closed");
   exception
      when
            Directory_Error =>
         Raise_Exception(Listing_Error'Identity, "Probably invalid filename " & To_String(Directory_Name) & " & " & To_String(Internal_Directory_Name));
   end;

   --
   -- Is_Directory
   --

   function Is_Directory (
         Name : String ) 
     return Boolean is 
   begin
      return Gnat.Os_Lib.Is_Directory(Name);
   end;


end Directory_Listings;

