with Access_Rights,
   Gnat.Os_Lib,--semi-native functions :-)
   Directory_Deletions,--native functions
   Gnat.Directory_Operations,
   Paskma.Logger,
   Ada.Characters.Handling,
   Ada.Strings.Fixed,
   Directory_Listings,--native functions
   Directory_Listing_Utilities,
   Ada.Exceptions,
   File_Utilities,
   Paskma.String_Utilities,
   Paskma.Lists;

use Access_Rights,
   Paskma.Logger,
   Ada.Characters.Handling,
   Ada.Strings.Fixed,
   Directory_Listings,
   Directory_Listing_Utilities,
   Ada.Exceptions,
   File_Utilities,
   Paskma.String_Utilities;

package body File_Trees is

   procedure Initialize (
         Obj : in out File_Tree_Type ) is 
   begin
      Uninitialize(Obj); -- really humorous :-)
   end;

   procedure Uninitialize (
         Obj : in out File_Tree_Type ) is 
   begin
      Empty(Obj.Directories);
      Empty(Obj.Mapping);
      Obj.Root_Directory := To_Unbounded_String("");
      Obj.Inited := False;
      Obj.Filesystem_Case_Sensitive := True;
   end;

   --
   -- Construct
   --

   procedure Construct (
         -- should be named Initialize
         -- the only method makes object initialized
         Obj                          : in out File_Tree_Type;        
         Directories_For_Current_User :        Shared_Dir_Lists.List; 
         Root_Directory               :        Unbounded_String;      
         Mapping                      :        Mapping_Lists.List;    
         Filesystem_Case_Sensitive    :        Boolean                ) is 

   begin
      Uninitialize(Obj);
      Obj.Directories := Directories_For_Current_User;
      Obj.Root_Directory := Root_Directory;
      Obj.Mapping := Mapping;
      Obj.Inited := True;
      Obj.Current_Directory := To_Unbounded_String("/");
      Obj.Filesystem_Case_Sensitive := Filesystem_Case_Sensitive;
   end;

   --
   -- Translate_To_Physical
   --
   -- Returns the absolut physical filename/path
   -- -- Investigate_New_Root is its slave

   procedure Investigate_New_Root (
         A_List                    :        Mapping_Lists.List; 
         Alias                     :        Unbounded_String;   
         Filesystem_Case_Sensitive :        Boolean;            
         Root_Changed              :    out Boolean;            
         New_Root                  : in out Unbounded_String    ) is 

      I     :          Mapping_Lists.Iterator := Get_Begin (A_List);  
      I_End : constant Mapping_Lists.Iterator := Get_End (A_List);  
   begin

      -- If the filesystem is case INsensitive then the path
      -- /MeDia/the_film.avi
      -- must be translated to
      -- c:\media\the_film.avi
      -- If it is SENSITIVE, it must NOT!

      if Filesystem_Case_Sensitive then

         while I /= I_End loop
            if Alias = Dereference(I).Alias then -- case sensitive comparision
               -- if the aliases are the same then change the physical root directory
               New_Root := Dereference(I).Directory;
               Root_Changed := True;
               return;
            end if;
            Increment(I);
         end loop;

      else
         --
         -- Filesystem is case insensitive

         while I /= I_End loop
            if Insensitive_Compare(Alias, Dereference(I).Alias) then -- case INsensitive comparision
               -- if the aliases are the same then change the physical root directory
               New_Root := Dereference(I).Directory;
               Root_Changed := True;
               return;
            end if;
            Increment(I);
         end loop;


      end if;

      -- keep New_Root unchanged
      Root_Changed := False;
   end Investigate_New_Root;

   --
   -- master itself
   --

   function Translate_To_Physical (
         Obj              : in     File_Tree_Type;  
         Logical_Filename :        Unbounded_String ) 
     return Unbounded_String is 

      Absolut_Logical_Name,  
      Work_Logical_Filename,  
      Real_Root             : Unbounded_String;  
      Root_Changed          : Boolean;  
      Result                : Unbounded_String;  
   begin

      if Logical_Filename = Null_Unbounded_String then
         raise Invalid_Filename;
      end if;

      --
      -- Remove the dot from the mc's "./" notation
      Work_Logical_Filename := Logical_Filename;
      if Length(Logical_Filename) >= 2 and then Slice(Logical_Filename, Length(Logical_Filename)-1,Length(Logical_Filename)) = "/." then
         Work_Logical_Filename := +Slice(Logical_Filename, 1, Length(Logical_Filename)-1);
      else
         Work_Logical_Filename := Logical_Filename;
      end if;

      Absolut_Logical_Name := Get_Absolut_Name(Obj, Work_Logical_Filename);

      Real_Root := Obj.Root_Directory;
      Investigate_New_Root(
         Obj.Mapping,
         Get_First_Path_Element(Absolut_Logical_Name),
         -- the first path element is used as potentional alias
         Obj.Filesystem_Case_Sensitive,
         Root_Changed,
         Real_Root);

      if Root_Changed then
         Result := Compose_Path(Real_Root, Logical_To_Physical_Separator(Cut_First_Path_Element(Absolut_Logical_Name)));
         -- if the root was changed, we must cut off the first path element ("alias")
      else
         Result := Compose_Path(Real_Root, Logical_To_Physical_Separator(Absolut_Logical_Name));
      end if;

      --Log("Translated """ & Logical_Filename & """ to """ & Result);
      return Result;

   end Translate_To_Physical;

   --
   -- Remove_Directory_Dots
   --
   -- solves shit as /media/../media/../media/././mp3/abba

   function Remove_Directory_Dots (
         Absolut_Logical_Filename : Unbounded_String ) 
     return Unbounded_String is 
      use String_Lists;
      Parts,  
      Result_Parts : String_Lists.List;  
      I,  
      I_Begin,  
      I_End        : String_Lists.Iterator;  
      Skip_Count   : Natural               := 0;  
      Result       : Unbounded_String;  
   begin
      Get_Elements(Absolut_Logical_Filename, '/',  Parts);
      --parse to parts

      I_Begin := Get_Begin(Parts);
      I := Get_End(Parts);

      --
      -- If there is no parts, return the root directory
      if Length(Parts) = 0 then
         return +"/";
      end if;

      --
      -- Process all parts backward (because of ".." skiping) and handle "." and ".."
      loop
         Decrement(I);

         if Dereference(I) = +"." then --ignore dots
            null;
         else
            if Dereference(I) = +".." then --each ".." causes one skip
               Skip_Count := Skip_Count + 1;
            else
               --
               -- Skip or insert to result
               if Skip_Count = 0 then
                  Push_Front(Dereference(I), Result_Parts);
               else
                  Skip_Count := Skip_Count - 1;
               end if;
            end if;
         end if;

         exit when I = I_Begin;
      end loop;

      --
      -- if we are higher than in the root directory => return root
      if Skip_Count /= 0 then
         return +"/";
      end if;

      --
      -- Compose back
      I := Get_Begin(Result_Parts);
      I_End := Get_End(Result_Parts);
      while I /= I_End loop
         Result := Result & '/' & Dereference(I);
         Increment(I);
      end loop;

      if Length(Result) = 0 then
         Result := +"/";
      end if;

      return Result;
   end;

   --
   -- Get_Absolut_Name
   --

   function Get_Absolut_Name (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ) 
     return Unbounded_String is 

      Absolut_Logical_Name : Unbounded_String; -- result                                                                                                                  
   begin
      if Length(Logical_Filename) = 0 then
         -- if  the argument is "", current directory is returned
         return Get_Current_Directory(Obj);
      end if;
      --
      -- determine the full (absolut) logical path
      if Element(Logical_Filename, 1) = '/' then
         -- the given name is absolut
         Absolut_Logical_Name := Logical_Filename;
      else
         -- the given name is relative
         Absolut_Logical_Name := Compose_Logical_Path(Obj.Current_Directory, Logical_Filename);
      end if;

      --Log(Absolut_Logical_Name & " => " & Remove_Directory_Dots(Absolut_Logical_Name));
      return Remove_Directory_Dots(Absolut_Logical_Name);
   end Get_Absolut_Name;


   --
   -- Get_Current_Directory
   --

   function Get_Current_Directory (
         Obj : File_Tree_Type ) 
     return Unbounded_String is 
   begin
      return Display_Directory_Name(Obj.Current_Directory);
   end Get_Current_Directory;

   --
   -- Change_Directory
   --

   procedure Change_Directory (
         Obj            : in out File_Tree_Type;  
         Directory_Name :        Unbounded_String ) is 

      Name_To_Test : Unbounded_String;  
   begin
      if Length(Directory_Name) = 0 then
         raise Invalid_Filename;
      end if;

      --      if Element(Directory_Name, 1) = '/' then
      --         -- name is absolut
      --         Name_To_Test := Directory_Name;
      --      else
      --         -- name is relative
      --         Name_To_Test := Compose_Logical_Path(Obj.Current_Directory, Directory_Name);
      --      end if;

      Name_To_Test := Get_Absolut_Name(Obj, Directory_Name);

      if Directory_Listings.Is_Directory(To_String(Translate_To_Physical(Obj, Name_To_Test))) then
         --_____Gnat.Os_Lib.Is_Directory(To_String(Translate_To_Physical(Obj, Name_To_Test))) 
         -- new directory name is ok
         Obj.Current_Directory := Name_To_Test; -- and tested
      else
         raise Invalid_Filename;
      end if;
   end Change_Directory;

   --
   -- Change_Directory_Up
   --

   procedure Change_Directory_Up (
         Obj : in out File_Tree_Type ) is 
      Directory_Name : Unbounded_String;  
   begin
      Directory_Name := Get_Current_Directory(Obj);

      if Directory_Name = "/" then
         -- when hit the wall, do not injure head :-)
         return;
      end if;

      Directory_Name := Cut_Last_Path_Element(Directory_Name);
      -- here we switching up
      Change_Directory(Obj, Directory_Name);
   end Change_Directory_Up;

   --
   --  Verify_Access_Rights
   --
   -- raises access_not_allowed if needed

   procedure Verify_Access_Rights (
         Obj              : File_Tree_Type;   
         Logical_Filename : Unbounded_String; 
         Demanded_Right   : Access_Right_Type ) is 

      I,  
      I_End         : Shared_Dir_Lists.Iterator;  
      Physical_Name : Unbounded_String;  
   begin
      I := Get_Begin(Obj.Directories);
      I_End := Get_End(Obj.Directories);

      Physical_Name := Translate_To_Physical(Obj, Logical_Filename);

      if Demanded_Right /= Directory_List then
         Physical_Name := Cut_Last_Physical_Path_Element(Physical_Name);
         -- If the physical_name is a name of FILE, it will be truncated an the DIRECTORY name will be compared.
         -- If the physical_Name is a name of DIRECTORY, the name of parent directory will be compared.
      end if;

      while I /= I_End loop
         if Obj.Filesystem_Case_Sensitive then
            --
            -- we are running on u*x system
            if Index(Physical_Name, To_String(Dereference(I).Name)) = 1 then
               -- the asked filename must begin with one of the strings in obj.directories

               if Is_Included(Dereference(I).Rights, Demanded_Right) then
                  return;
               else
                  raise Access_Not_Allowed;
               end if;

            end if;
         else
            --
            -- windows
            if Index(To_Upper(To_String(Physical_Name)), To_Upper(To_String(Dereference(I).Name))) = 1 then
               -- the asked filename must begin with one of the strings in obj.directories

               if Is_Included(Dereference(I).Rights, Demanded_Right) then
                  return;
               else
                  raise Access_Not_Allowed;
               end if;

            end if;
         end if;
         Increment(I);
      end loop;

      raise Access_Not_Allowed;
   end;


   --
   -- Get_Directory_Listing
   --

   procedure Get_Directory_Listing (
         Obj            :        File_Tree_Type;                      
         Directory_Name :        Unbounded_String;                    
         Result         : in out Unbounded_String_Lists.List;         
         Names_Only     :        Boolean                     := False ) is 

      Listing_Result   : Directory_Item_Lists.List;  
      I,  
      I_End            : Directory_Item_Lists.Iterator;  
      Listing_Dir_Name : Unbounded_String;  

      use Directory_Item_Lists;
   begin
      Empty(Result);

      if Directory_Name = Null_Unbounded_String then
         -- if no directory is given then listing the current directory
         Listing_Dir_Name := Obj.Current_Directory;
      else
         --
         -- Some clients put nonstandard parameters (-al etc) as LIST parameter
         if Element(Directory_Name, 1) = '-' then
            --
            -- And these parameters are ignored (in fact it is not very elegant)

            Log("Warning: probably attempt to pass parameter " & Directory_Name);
            declare
               Space_Index : Integer := Index (Directory_Name, " ");  
               -- there is a space behind the attribute (?)
            begin
               if Space_Index = 0 then
                  Listing_Dir_Name := Obj.Current_Directory;
               else
                  Listing_Dir_Name := Tail(Directory_Name, Length(Directory_Name) - Space_Index);
               end if;
            end;
         else
            --
            -- if ordinary directory name was found => use it
            Listing_Dir_Name := Directory_Name;
         end if;
      end if;

      --Log("Verifing acces rights for: " & Listing_Dir_Name);
      Verify_Access_Rights(Obj, Listing_Dir_Name, Directory_List);
      -- when access is not allowed then exception is raised

      Get_Listing(Translate_To_Physical(Obj, Listing_Dir_Name), Listing_Result);

      --
      -- translate the listing result to the string form
      I := Get_Begin(Listing_Result);
      I_End := Get_End(Listing_Result);

      while I /= I_End loop
         if Names_Only then
            Push_Back(Dereference(I).Name, Result);
         else
            Push_Back(To_String(Dereference(I)), Result);
         end if;
         Increment(I);
      end loop;

      --
      -- List mapped aliases (the "/." syntax using mc)
      if Listing_Dir_Name = To_Unbounded_String("/") or Listing_Dir_Name = To_Unbounded_String("/.") then
         -- if we are listing the root directory then assume mapped directories
         declare
            Found_Item    : Directory_Item_Type;  
            Mapping_I,  
            Mapping_I_End : Mapping_Lists.Iterator;  
         begin
            Mapping_I := Get_Begin(Obj.Mapping);
            Mapping_I_End := Get_End(Obj.Mapping);

            -- get listing for all mapped directories
            while Mapping_I /= Mapping_I_End loop
               begin
                  --
                  -- this Get_Listing is called only to determine attributes
                  Get_Listing(
                     Display_Directory_Name( -- to cut the end backslash
                        Translate_To_Physical(
                           Obj,
                           Compose_Logical_Path(
                              -- create the absolut logical path of mapped directory
                              To_Unbounded_String("/"),
                              Dereference(Mapping_I).Alias))),
                     Listing_Result,
                     False);

                  --
                  -- There is physical name in the found_item, but _
                  -- we want user to see aliases.
                  Found_Item := Dereference(Get_Begin(Listing_Result));
                  Found_Item.Name := Dereference(Mapping_I).Alias;

                  if Names_Only then
                     Push_Back(Found_Item.Name, Result);
                  else
                     Push_Back(To_String(Found_Item), Result);
                  end if;

                  Increment(Mapping_I);
               exception
                  when Event : Listing_Error => --probably errror in mapping configuration caused invalid filename here
                     Log("Exception attribute recognition ");
                     Log(Exception_Information(Event));
                     Increment(Mapping_I);--just continue in iterating
               end;
            end loop;
         end;
      end if;

   exception
      when Access_Not_Allowed =>
         null;
      when Event : Listing_Error =>
         Log("Exception during listing ");
         Log(Exception_Information(Event));
   end Get_Directory_Listing;

   --
   -- Open_File_read
   --

   procedure Open_File_Read (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
   begin
      Verify_Access_Rights(Obj, Logical_Filename, File_Read);
      -- exception is raised when access is not allowed
      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);
      Open(File, In_File, To_String(Physical_Filename), "shared=yes");
   exception
      when Name_Error | Use_Error =>
         --Log(Exception_Information(Event));
         raise Invalid_Filename;
   end Open_File_Read;

   --
   -- Open_File_Write
   --

   procedure Open_File_Write (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
   begin
      Verify_Access_Rights (Obj, Logical_Filename, File_Write);
      -- exception is raised when access_is_not_allowed

      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);

      if Gnat.Os_Lib.Is_Regular_File(To_String(Physical_Filename & Character'First)'Address) then
         -- if demanded file already exists
         Verify_Access_Rights(Obj, Logical_Filename, File_Delete);
         -- assure if we can delete (overwrite) it
         Open(File, Out_File, To_String(Physical_Filename));
         -- open for overwriting
      else
         Create(File, Out_File, To_String(Physical_Filename));
         -- create a new file
      end if;

   exception
      when Name_Error | Use_Error =>
         raise Invalid_Filename;
   end Open_File_Write;

   --
   -- Open_File_Read_Write
   --

   procedure Open_File_Read_Write (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
   begin
      --
      -- This provides full control over demanded file
      Verify_Access_Rights (Obj, Logical_Filename, File_Read);
      Verify_Access_Rights (Obj, Logical_Filename, File_Write);
      Verify_Access_Rights (Obj, Logical_Filename, File_Delete);
      -- exception is raised when access_is_not_allowed

      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);

      Open(File, In_File, To_String(Physical_Filename));
      -- the file mode is set to in_file to avoid deleting file. 
      -- The mode can be changed by Set_Mode

   exception
      when Name_Error | Use_Error =>
         raise Invalid_Filename;
   end Open_File_Read_Write;

   --
   -- Open_File_Append
   --

   procedure Open_File_Append (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
   begin
      Verify_Access_Rights (Obj, Logical_Filename, File_Append);
      -- exception is raised when access_is_not_allowed

      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);

      if Gnat.Os_Lib.Is_Regular_File(To_String(Physical_Filename & Character'First)'Address) then
         -- if demanded file already exists
         Open(File, Append_File, To_String(Physical_Filename));
         -- open for appending
      else
         Create(File, Out_File, To_String(Physical_Filename));
         -- create a new file
      end if;

   exception
      when Name_Error | Use_Error =>
         raise Invalid_Filename;
   end Open_File_Append;

   --
   -- Delete_File
   --

   procedure Delete_File (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
      Success           : Boolean;  
   begin
      Verify_Access_Rights(Obj, Logical_Filename, File_Delete);

      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);

      Gnat.Os_Lib.Delete_File(To_String(Physical_Filename), Success);

      if not Success then
         raise Invalid_Filename;
      end if;
   end Delete_File;


   --
   -- Create_Directory
   --

   procedure Create_Directory (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
   begin
      Verify_Access_Rights(Obj, Logical_Filename, Directory_Create);
      -- exception is propagated when access_is_not_allowed
      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);
      Gnat.Directory_Operations.Make_Dir(To_String(Physical_Filename));
   exception
      when Gnat.Directory_Operations.Directory_Error =>
         raise Invalid_Filename;
   end Create_Directory;

   --
   -- Delete_Directory
   --

   procedure Delete_Directory (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ) is 

      Physical_Filename : Unbounded_String;  
      Success           : Boolean;  
   begin
      Verify_Access_Rights(Obj, Logical_Filename, Directory_Delete);

      Physical_Filename := Translate_To_Physical(Obj, Logical_Filename);

      Success := Directory_Deletions.Delete_Directory(Physical_Filename);

      if not Success then
         raise Invalid_Filename;
      end if;
   end Delete_Directory;

   --
   -- Rename_File_Or_Directory
   --

   procedure Rename_File_Or_Directory (
         Obj                  : File_Tree_Type;  
         Old_Logical_Filename,                   
         New_Logical_Filename : Unbounded_String ) is 

      Success               : Boolean;  
      Physical_Old_Filename,  
      Physical_New_Filename : Unbounded_String;  
   begin
      Verify_Access_Rights(Obj, Old_Logical_Filename, File_Read);
      Verify_Access_Rights(Obj, Old_Logical_Filename, File_Delete);
      Verify_Access_Rights(Obj, Old_Logical_Filename, Directory_Delete);
      Verify_Access_Rights(Obj, Old_Logical_Filename, Directory_List);
      Verify_Access_Rights(Obj, New_Logical_Filename, File_Write);
      Verify_Access_Rights(Obj, New_Logical_Filename, Directory_Create);
      -- TODO: currently: Access rights must be allowed for both directories and file to rename.

      Physical_Old_Filename := Translate_To_Physical(Obj, Old_Logical_Filename);
      Physical_New_Filename := Translate_To_Physical(Obj, New_Logical_Filename);

      Gnat.Os_Lib.Rename_File(
         To_String(Physical_Old_Filename),
         To_String(Physical_New_Filename),
         Success);

      if not Success then
         raise Invalid_Filename;
      end if;
   end Rename_File_Or_Directory;


end File_Trees;