with Ada.Finalization,
   Ada.Strings.Unbounded,
   Paskma.Lists,
   Ada.Streams.Stream_Io,
   Configuration_Mycelium;

use Ada.Finalization,
   Ada.Strings.Unbounded,
   Ada.Streams.Stream_Io,
   Configuration_Mycelium;

package File_Trees is

   package Unbounded_String_Lists is new Paskma.Lists(Unbounded_String);

   use Unbounded_String_Lists;

   use Shared_Dir_Lists;
   use Mapping_Lists;

   Invalid_Filename   : exception;  
   Access_Not_Allowed : exception;  

   type File_Tree_Type is new Controlled with private; 

   -----------------------------------------------------------------------

   procedure Uninitialize (
         -- makes the file tree unusable
         Obj : in out File_Tree_Type ); 

   -----------------------------------------------------------------------

   procedure Construct (
         Obj                          : in out File_Tree_Type;        
         Directories_For_Current_User :        Shared_Dir_Lists.List; 
         Root_Directory               :        Unbounded_String;      
         Mapping                      :        Mapping_Lists.List;    
         Filesystem_Case_Sensitive    :        Boolean                ); 

   -----------------------------------------------------------------------

   function Get_Current_Directory (
         Obj : File_Tree_Type ) 
     return Unbounded_String; 

   -----------------------------------------------------------------------

   function Get_Absolut_Name (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ) 
     return Unbounded_String; 

   -----------------------------------------------------------------------

   procedure Change_Directory (
         Obj            : in out File_Tree_Type;  
         Directory_Name :        Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Change_Directory_Up (
         Obj : in out File_Tree_Type ); 

   -----------------------------------------------------------------------

   procedure Get_Directory_Listing (
         Obj            :        File_Tree_Type;                      
         Directory_Name :        Unbounded_String;                    
         Result         : in out Unbounded_String_Lists.List;         
         Names_Only     :        Boolean                     := False ); 

   -----------------------------------------------------------------------

   procedure Open_File_Read (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Open_File_Write (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Open_File_Read_Write (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Open_File_Append (
         Obj              :        File_Tree_Type;  
         File             :    out File_Type;       
         Logical_Filename :        Unbounded_String ); 

   -----------------------------------------------------------------------
   procedure Delete_File (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Create_Directory (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Delete_Directory (
         Obj              : File_Tree_Type;  
         Logical_Filename : Unbounded_String ); 

   -----------------------------------------------------------------------

   procedure Rename_File_Or_Directory (
         Obj                  : File_Tree_Type;  
         Old_Logical_Filename,                   
         New_Logical_Filename : Unbounded_String ); 

   -----------------------------------------------------------------------


private
   type File_Tree_Type is new Controlled with 
      record 
         Inited                    : Boolean;  
         Directories               : Shared_Dir_Lists.List;  
         Mapping                   : Mapping_Lists.List;  
         Filesystem_Case_Sensitive : Boolean;  
         -- these are not the same as in the configuration. These are allowed for current user
         Root_Directory,  
         -- physical, logical root directory is "/", of course
         Current_Directory : Unbounded_String;  
         -- logical
      end record; 

   procedure Initialize (
         -- this declaration must be here because of <i>controlled</i> system
         Obj : in out File_Tree_Type ); 

end File_Trees;