with Ada.Finalization,
   Ada.Strings.Unbounded,
   Gnat.Sockets,
   Configuration_Mycelium,
   File_Trees;

use Ada.Finalization,
   Ada.Strings.Unbounded,
   Gnat.Sockets,
   Configuration_Mycelium,
   File_Trees;

package Ftp_Configurations is

   type Ftp_Configuration_Type is new Controlled with private; 

   Access_Not_Allowed,  
   Configuration_Not_Loaded : exception;  

   procedure Load_From_Directory (
         Conf      : in out Ftp_Configuration_Type; 
         Directory :        Unbounded_String        ); 

   procedure Log (
         Conf : Ftp_Configuration_Type ); 

   function Get_Ip (
         Conf : Ftp_Configuration_Type ) 
     return Inet_Addr_Type; 

   function Get_Port (
         Conf : Ftp_Configuration_Type ) 
     return Port_Type; 

   procedure Verify_User (
         Conf     : Ftp_Configuration_Type; 
         Username : Unbounded_String        ); -- exception if failed

   procedure Verify_User (
         Conf     : Ftp_Configuration_Type; 
         Username,                          
         Password : Unbounded_String        ); -- exception if failed

   function Get_File_Tree (
         Conf     : Ftp_Configuration_Type; 
         Username,                          
         Password : Unbounded_String        ) 
     return File_Tree_Type; 

private
   type Ftp_Configuration_Type is new Controlled with 
      record 
         Loaded                    : Boolean;  
         Ip                        : Inet_Addr_Type;  
         Port                      : Port_Type;  
         Filesystem_Case_Sensitive : Boolean;  
         Accounts                  : User_Conf_Lists.List;  
         Mapping                   : Mapping_Lists.List;  
      end record; 

   procedure Initialize (
         Conf : in out Ftp_Configuration_Type ); -- controlled type 

   procedure Test_Loaded (
         Conf : Ftp_Configuration_Type ); 

end Ftp_Configurations;


