with Paskma.Logger,
   Socket_Utilities,
   Ada.Exceptions,
   Ftp_Protocol_Interpreter;

use Paskma.Logger,
   Socket_Utilities, -- for Read_Line only
   Ada.Exceptions,
   Ftp_Protocol_Interpreter;


package body Ftp_Server.Session is


   -----------------------
   -- Session_Task_Type --
   -----------------------

   task body Session_Task_Type is
      Accepting_Socket,  
      Pi_Socket        : Socket_Type;  
      Parent           : Parent_Shortcut_Type;  
      Task_Id          : Task_Id_Type;  
      Pi_Address       : Sock_Addr_Type;  
      Configuration    : Ftp_Configuration_Type;  
      -- Note: "PI" means protocol interpreter as mentioned in rfc 959
      Pi_Context : Pi_Context_Type;  
      Line       : Unbounded_String;  
      Quit       : Boolean;  
      Statistics : Statistics_Manager_Access;  
   begin
      accept Start (
            A_Configuration            : Ftp_Configuration_Type;   
            Accept_On_Socket           : Socket_Type;              
            Parent_For_New_Sessions    : Parent_Shortcut_Type;     
            Some_Task_Id               : Task_Id_Type;             
            Store_Statiscs_Information : Statistics_Manager_Access ) do 

         Configuration := A_Configuration;
         Accepting_Socket := Accept_On_Socket;
         Parent := Parent_For_New_Sessions;
         Task_Id := Some_Task_Id;
         Statistics := Store_Statiscs_Information;
      end Start;
      Log(Task_Id, "The beginning of new life (task).");

      Accept_Socket(Accepting_Socket, Pi_Socket, Pi_Address);

      Log(Task_Id, "Accepted connection from: [" & Image(Pi_Address) & "]");

      Create_New_Session(Parent);
      -- this session is accepted => create new one to wait for another client
      Construct(Pi_Context,
         Configuration,
         Pi_Socket,
         Task_Id,
         Pi_Address,
         Statistics);
      -- construct the context for the message loop
      Write_Welcome_Message(Pi_Context);

      --
      -- The message loop
      loop
         Line := Read_Line(Pi_Socket);
         exit when Line = To_Unbounded_String("");
         Log(Task_Id, "Line: " & Line);
         -- socket might be closed
         Dispatch_Message(Pi_Context, Line, Quit);
         exit when Quit;
      end loop;

      begin
         Statistics.Log_Out_User(Task_Id);
         -- tidy up for the case when the pi-chanell was  closed
      exception
         when Item_Not_Found =>
            null;
      end;

      Log(Task_Id, "Properly task end.");
   exception
      when Socket_Error =>
         Log(Task_Id, "Socket was probably closed by peer. Task end.");
         begin
            Statistics.Log_Out_User(Task_Id);
            -- tidy up for the case when the pi-chanell was  closed
         exception
            when Item_Not_Found =>
               null;
         end;
      when Event : others =>
         Log(Task_Id, Exception_Information(Event));
         Log(Task_Id, "Error task end.");

   end Session_Task_Type;

end Ftp_Server.Session;

