with Socket_Utilities,
   Ada.Strings.Maps,
   Ada.Strings;

use Socket_Utilities,
   Ada.Strings.Maps,
   Ada.Strings;

package body Pi_Conversion_Layer is

   ----------------
   -- Send_Reply --
   ----------------

   procedure Send_Reply (
         To_Socket   : Socket_Type;     
         Of_Code     : Reply_Code_Type; 
         And_Message : Unbounded_String ) is 

      Reply : Unbounded_String;  
   begin
      Reply := Image(Of_Code) & " " & And_Message;
      Write_Line(To_Socket, Reply);
   end Send_Reply;

   procedure Send_Reply (
         To_Socket   : Socket_Type;     
         Of_Code     : Reply_Code_Type; 
         And_Message : String           ) is 

      Reply : Unbounded_String;  
   begin
      Reply := Image(Of_Code) & " " & And_Message;
      --Log(obj.task_id, Reply);
      Write_Line(To_Socket, Reply);
   end Send_Reply;

   --
   -- Log
   --

   procedure Log (
         Task_Id : Task_Id_Type;    
         Code    : Reply_Code_Type; 
         Message : String           ) is 

      Reply : Unbounded_String;  
   begin
      Reply := Image(Code) & " " & Message;
      Log(Task_Id, Reply);
   end Log;

   procedure Log (
         Task_Id : Task_Id_Type;    
         Code    : Reply_Code_Type; 
         Message : Unbounded_String ) is 

      Reply : Unbounded_String;  
   begin
      Reply := Image(Code) & " " & Message;
      Log(Task_Id, Reply);
   end Log;


   --
   -- Parse
   --

   procedure Parse (
         Line : Unbounded_String; 
         -- no CRLF at the end
         Command   :    out Command_Type;    
         Parameter :    out Unbounded_String ) is 

      Command_End : Natural;  
   begin
      Command_End := Index(Line, " ");

      if Command_End /= 0 then
         Command := To_Command(To_Unbounded_String(Slice(Line, 1,Command_End-1)));
         Parameter := To_Unbounded_String(Slice(Line, Command_End+1, Length(Line)));
      else
         Command := To_Command(Line);
         Parameter := To_Unbounded_String("");
      end if;

   exception
      when Constraint_Error | Index_Error =>
         raise Invalid_Command;
   end Parse;

   --
   -- Ftp_Address_to_socket_address
   --

   function Ftp_Address_To_Socket_Address (
         Ftp_Addr : Unbounded_String ) 
     return Sock_Addr_Type is 

      Internal_Ftp_Addr : Unbounded_String;  
      Result            : Sock_Addr_Type;  
      Tmp_Port,  
      Last_Comma_Pos    : Natural;  
   begin
      if Count(Ftp_Addr, ",") /= 5 or Length(Ftp_Addr) < 11 then
         raise Constraint_Error;
      end if;

      --
      -- low-order byte of port
      Last_Comma_Pos :=  Index(Ftp_Addr, ",", Backward);
      Tmp_Port := Integer'Value(Slice(Ftp_Addr, Last_Comma_Pos+1, Length(Ftp_Addr)));

      --
      -- high order byte of port
      Internal_Ftp_Addr := Head(Ftp_Addr, Last_Comma_Pos-1);
      -- replace the last element (low order byte of port)
      Last_Comma_Pos :=  Index(Internal_Ftp_Addr, ",", Backward);
      Tmp_Port := Tmp_Port + 256 * Integer'Value(Slice(Internal_Ftp_Addr, Last_Comma_Pos+1, Length(Internal_Ftp_Addr)));
      Internal_Ftp_Addr := Head(Internal_Ftp_Addr, Last_Comma_Pos-1);
      -- replace the last element again (high order byte of port)

      Translate(Internal_Ftp_Addr, To_Mapping(",", "."));
      -- 127,0,0,1 => 127.0.0.1

      Result.Addr := Inet_Addr(To_String(Internal_Ftp_Addr));
      Result.Port := Port_Type(Tmp_Port);

      return Result;
   exception
      when Index_Error =>
         raise Constraint_Error;
   end;

   function Socket_Address_To_Ftp_Address (
         Socket_Address : Sock_Addr_Type ) 
     return Unbounded_String is 

      Address_Part : Unbounded_String;  
   begin
      Address_Part := To_Unbounded_String(Image(Socket_Address.Addr));
      Translate(Address_Part, To_Mapping(".", ","));

      return Address_Part &
         "," &
         Trim(To_Unbounded_String(Integer'Image(Integer(Socket_Address.Port) / 256)), Left) &
         "," &
         Trim(To_Unbounded_String(Integer'Image(Integer(Socket_Address.Port) mod 256)), Left);
   end;

end Pi_Conversion_Layer;

