with Ada.Strings;

use Ada.Strings;

package body Reply_Codes is

   --
   -- these internal versions are here because we want backward checks
   -- and we don't want infinite recursion

   function Internal_To_Integer (
         Code : Reply_Code_Type ) 
     return Integer; 

   function Internal_To_Reply_Code (
         Num : Integer ) 
     return Reply_Code_Type; 


   -----------
   -- Image --
   -----------

   function Image (
         Code : Reply_Code_Type ) 
     return Unbounded_String is 
   begin
      return Trim(To_Unbounded_String(Integer'Image(To_Integer(Code))),
         Left);
   end Image;

   ----------------
   -- To_Integer --
   ----------------
   -- and the internal version

   function Internal_To_Integer (
         Code : Reply_Code_Type ) 
     return Integer is 
   begin
      case Code is
         when The_110 =>
            return 110;
         when The_120=>
            return 120;
         when The_125 =>
            return 125;
         when The_150 =>
            return 150;
         when The_200 =>
            return 200;
         when The_202 =>
            return 202;
         when The_211 =>
            return 211;
         when The_212 =>
            return 212;
         when The_213 =>
            return 213;
         when The_214 =>
            return 214;
         when The_215 =>
            return 215;
         when The_220 =>
            return 220;
         when The_221 =>
            return 221;
         when The_225 =>
            return 225;
         when The_226 =>
            return 226;
         when The_227 =>
            return 227;
         when The_230 =>
            return 230;
         when The_231 =>
            return 231;
         when The_250 =>
            return 250;
         when The_257 =>
            return 257;
         when The_331 =>
            return 331;
         when The_332 =>
            return 332;
         when The_350 =>
            return 350;
         when The_421 =>
            return 421;
         when The_425 =>
            return 425;
         when The_426 =>
            return 426;
         when The_450 =>
            return 450;
         when The_451 =>
            return 451;
         when The_452 =>
            return 452;
         when The_500 =>
            return 500;
         when The_501 =>
            return 501;
         when The_502 =>
            return 502;
         when The_503 =>
            return 503;
         when The_504 =>
            return 504;
         when The_530 =>
            return 530;
         when The_532 =>
            return 532;
         when The_550 =>
            return 550;
         when The_551 =>
            return 551;
         when The_553=>
            return 553;
         when others =>
            raise Constraint_Error;
      end case;
   end Internal_To_Integer;

   --
   -- the public master itself

   function To_Integer (
         Code : Reply_Code_Type ) 
     return Integer is 

      Result : Integer;  
   begin
      Result := Internal_To_Integer(Code);
      if Internal_To_Reply_Code(Result) /= Code then
         -- military backward check
         raise Program_Error;
      end if;

      return Result;
   end;

   -------------------
   -- To_Reply_Code --
   -------------------
   -- and the internal version

   function Internal_To_Reply_Code (
         Num : Integer ) 
     return Reply_Code_Type is 
   begin
      case Num is
         when 110 =>
            return The_110;
         when 120=>
            return The_120;
         when 125 =>
            return The_125;
         when 150 =>
            return The_150;
         when 200 =>
            return The_200;
         when 202 =>
            return The_202;
         when 211 =>
            return The_211;
         when 212 =>
            return The_212;
         when 213 =>
            return The_213;
         when 214 =>
            return The_214;
         when 215 =>
            return The_215;
         when 220 =>
            return The_220;
         when 221 =>
            return The_221;
         when 225 =>
            return The_225;
         when 226 =>
            return The_226;
         when 227 =>
            return The_227;
         when 230 =>
            return The_230;
         when 231 =>
            return The_231;
         when 250 =>
            return The_250;
         when 257 =>
            return The_257;
         when 331 =>
            return The_331;
         when 332 =>
            return The_332;
         when 350 =>
            return The_350;
         when 421 =>
            return The_421;
         when 425 =>
            return The_425;
         when 426 =>
            return The_426;
         when 450 =>
            return The_450;
         when 451 =>
            return The_451;
         when 452 =>
            return The_452;
         when 500 =>
            return The_500;
         when 501 =>
            return The_501;
         when 502 =>
            return The_502;
         when 503 =>
            return The_503;
         when 504 =>
            return The_504;
         when 530 =>
            return The_530;
         when 532 =>
            return The_532;
         when 550 =>
            return The_550;
         when 551 =>
            return The_551;
         when 553=>
            return The_553;
         when others =>
            raise Invalid_Reply_Code;
      end case;
   end Internal_To_Reply_Code;

   --
   -- the public master itself

   function To_Reply_Code (
         Num : Integer ) 
     return Reply_Code_Type is 

      Result : Reply_Code_Type;  
   begin
      Result := Internal_To_Reply_Code(Num);
      if Internal_To_Integer(Result) /= Num then
         -- military backward check
         raise Program_Error;
      end if;

      return Result;
   end;



end Reply_Codes;

