--with Paskma.Logger;
--use Paskma.Logger;

package body Socket_Utilities is

   --------------------------------------------------------------------------------

   --
   -- Sure_Send_Socket
   --
   -- Can be used for non-blocking sockets

   procedure Sure_Send_Socket (
         Socket : Socket_Type;         
         Buffer : Stream_Element_Array ) is 

      First,  
      Last  : Stream_Element_Offset;  
   begin
      First := Buffer'First;
      loop
         Send_Socket(Socket, Buffer(First..Buffer'Last), Last);
         exit when Last = Buffer'Last;
         -- exit when whole buffer was sent
         First := Last+1;
      end loop;
   end;

   --------------------------------------------------------------------------------

   function To_String (
         Buf  : Stream_Element_Array; 
         Last : Stream_Element_Offset ) 
     return Unbounded_String is 
      Result : Unbounded_String;  
   begin
      Result := To_Unbounded_String(Natural(Last));

      for I in Buf'First..Last loop
         Replace_Element(
            Result,
            Positive(I+Buf'First)-1,
            Character'Val(Buf(I))
            );
      end loop;

      return Result;
   end;

   --------------------------------------------------------------------------------

   function To_Stream (
         Buf : Unbounded_String ) 
     return Stream_Element_Array is 
      Result : Stream_Element_Array (1 .. Stream_Element_Offset (Length (Buf)));  
   begin
      for I in Result'range loop
         Result(I) := Character'Pos( Element(Buf, Integer(I-Result'First)+1) );
      end loop;
      return Result;
   end;

   --------------------------------------------------------------------------------

   procedure Copy_File_To_Socket (
         File   : Stream_Io.File_Type; 
         Socket : Socket_Type          ) is 

      Buffer_Size : constant Stream_Element_Offset                   := 65_536;  
      Buffer      :          Stream_Element_Array (1 .. Buffer_Size);  
      File_Last,  
      Socket_Last :          Stream_Element_Offset;  
   begin
      loop
         Read(File, Buffer, File_Last);

         Send_Socket(Socket, Buffer(1..File_Last), Socket_Last);

         if Socket_Last < File_Last then
            -- the whole LOADED buffer was not sent properly
            -- assuming blocking socket
            raise Socket_Error;
         end if;

         exit when End_Of_File(File);
      end loop;
   end;

   --------------------------------------------------------------------------------

   procedure Copy_Socket_To_File (
         Socket : Socket_Type;        
         File   : Stream_Io.File_Type ) is 

      Buffer_Size : constant Stream_Element_Offset                   := 65_536;  
      Buffer      :          Stream_Element_Array (1 .. Buffer_Size);  
      Socket_Last :          Stream_Element_Offset;  
   begin
      loop
         Receive_Socket(Socket, Buffer, Socket_Last);

         if Socket_Last = Buffer'First -1 then
            -- the socket was closed by peer
            exit;
         end if;

         Write(File, Buffer(1..Socket_Last));
      end loop;
   end;

   --------------------------------------------------------------------------------

   procedure Write_Line (
         Socket : Socket_Type;     
         Line   : Unbounded_String ) is 

      Last : Stream_Element_Offset;  
   begin
      Send_Socket(Socket, To_Stream(Line & Crlf), Last);
   end;

   --------------------------------------------------------------------------------

   function Read_Line (
         Socket : Socket_Type ) 
     return Unbounded_String is 

      Buffer : Stream_Element_Array (1 .. 65_536);  
      Last   : Stream_Element_Offset;  
      Result : Unbounded_String;  

      Max_Line_Length : constant := 10_000;  
   begin
      loop
         Receive_Socket(Socket, Buffer, Last);

         if Last = Buffer'First -1 then
            -- socket was probably closed by peer
            return To_Unbounded_String("");
         end if;

         Result := Result & To_String(Buffer, Last);

         if Slice(Result, Length(Result)-1, Length(Result)) = Crlf then
            -- line must be ended by crlf
            return To_Unbounded_String(Slice(Result, 1, Length(Result)-2));
            -- cut off the ending CrLf sequence
         elsif Length(Result) > Max_Line_Length then
            -- probably binary data
            raise Constraint_Error;
         end if;

      end loop;
   end;

end Socket_Utilities;

