with Ada.Strings.Unbounded,
   Gnat.Sockets,
   Paskma.Lists,
   Task_Id_Types;

use Ada.Strings.Unbounded,
   Gnat.Sockets,
   Task_Id_Types;

package Statistics is

   type Stat_Item_Type is 
      record 
         Username : Unbounded_String;  
         Task_Id  : Task_Id_Type;  
         Address  : Sock_Addr_Type;  
         Name,  
         Activity : Unbounded_String;  
      end record; 

   package Stat_Item_Lists is new Paskma.Lists(Stat_Item_Type);
   use Stat_Item_Lists;

   package String_Lists is new Paskma.Lists(Unbounded_String);
   use String_Lists;

   Item_Not_Found : exception;  


   protected type Statistics_Manager_Type is
      -------------------------------------------
      procedure Log_In_User (
            Username : Unbounded_String; 
            Task_Id  : Task_Id_Type;     
            Address  : Sock_Addr_Type    ); 
      -------------------------------------------
      procedure Log_Out_User (
            Task_Id : Task_Id_Type ); 
      -------------------------------------------
      procedure Set_Activity (
            Task_Id  : Task_Id_Type;    
            Activity : Unbounded_String ); 
      -------------------------------------------
      function Get_Info return String_Lists.List; 
      -------------------------------------------
      function Get_Info return Unbounded_String; 
      -------------------------------------------
      --
      -- Pseudo private funcions for name_completer_task
      --
      -------------------------------------------
      procedure Get_Unnamed_Ip (
            Success :    out Boolean;       
            Task_Id :    out Task_Id_Type;  
            Address :    out Sock_Addr_Type ); 
      -------------------------------------------
      procedure Set_Client_Name (
            Task_Id     : in     Task_Id_Type;    
            Client_Name :        Unbounded_String ); 
      -------------------------------------------
   private
      Items : aliased Stat_Item_Lists.List;

   end Statistics_Manager_Type;

   type Statistics_Manager_Access is access all Statistics_Manager_Type; 
   pragma Controlled(Statistics_Manager_Access);

   task type Name_Completer_Task_Type is
      entry Start (
            Statistics : Statistics_Manager_Access; 
            A_Delay    : Duration                   ); 
      entry Stop; 
   end Name_Completer_Task_Type;

end Statistics;
         